# Print a man page from the help texts.

# Author(s):
# clime <clime@fedoraproject.org>
#
# Original work:
# Copyright (C) 2011 Red Hat Inc.
# Author(s):
# Jesse Keating <jkeating@redhat.com>
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.  See http://www.gnu.org/copyleft/gpl.html for
# the full text of the license.


import os
import sys
import datetime


# We could substitute the "" in .TH with the rpkg version if we knew it
man_header = """\
.\\" man page for rpkg
.TH rpkg 1 "%(today)s" "" "rpm\\-packager"
.SH "NAME"
rpkg \\- RPM packaging utility
.SH "SYNOPSIS"
.B "rpkg"
[
.I global_options
]
.I "command"
[
.I command_options
]
[
.I command_arguments
]
.br
.B "rpkg"
.B "help"
.br
.B "rpkg"
.I "command"
.B "\\-\\-help"
.SH "DESCRIPTION"
.B "rpkg"
is a script to manage spec enriched Git repositories - meaning any repositories which contain
an rpm spec file. It is designed to work with expanded application sources as well as with standard DistGit
tarballs and patches. Note that you should trust the .spec files you work with because many operations
(like `rpkg srpm` or `rpkg lint`) involve parsing the spec file, which brings along evaluation
of any shell or lua scriplets present in it.
"""

man_footer = """\
.SH "CONFIGURATION"

rpkg reads configuration options from the following files in this order:

    /etc/rpkg.conf, ~/.config/rpkg.conf, <git_root>/rpkg.conf, ./rpkg.conf

Configuration found in ./rpkg.conf has the highest priority. Configuration
in /etc/rpkg.conf has the lowest priority. <git_root> placeholder above means
the top level of the Git repository we are working in. If we are in a plain
directory and all our parent directories are also plain, reading of
<git_root>/rpkg.conf is simply skipped.

.SS "DEFAULT COMMENTED CONFIGURATION"

    [rpkg]
    # whether to enable spec preprocessing,
    # that is parsing and evaluation of {{{ ... }}} expressions
    preprocess_spec = True

    # base directory where results should be stored
    base_output_path = /tmp/rpkg

    # path to user preproc macros
    # you may use ${git_props:root} variable
    # to anchor the path to toplevel of
    # of your git repo.
    user_macros_path =

    # optional path to rpmlint config
    # file to adjust linting of your
    # package. You may use ${git_props:root}
    # to anchor the path to toplevel
    # of your git repo.
    rpmlint_config =

    [git]
    # default Git repository URL, used by `rpkg clone <module>`
    clone_url = ssh://${user}@pkgs.fedoraproject.org/%(project)s

    # default anonymous Git repository URL, used by `rpkg clone -a <module>`
    anon_clone_url = https://src.fedoraproject.org/%(project)s

    [lookaside]
    # download URL for DistGit lookaside cache, used by `rpkg sources` command
    # %(ns1)s, %(name)s are parts of a module name (e.g. rpms/rpkg-util)
    # %(filename)s, %(hash_type)s, %(hash)s, %(filename)s fields are read from 'sources' file if present
    download_url = https://${git_pull_netloc}/repo/pkgs/%(ns1)s/%(name)s/%(filename)s/%(hash_type)s/%(hash)s/%(filename)s

    # upload URL for DistGit lookaside cache, used by `rpkg upload`
    upload_url = https://${git_push_netloc}/repo/pkgs/upload.cgi


.SH "EXAMPLES"

Run %prep phase of an rpm build:

    $ rpkg prep
    git_pack: packing directory /home/clime/rpkg-util
    git_pack: Wrote: /tmp/rpkg/rpkg-util-1-j8x35rb3/rpkg-util-2.2.git.18.5b6a884.tar.gz
    Wrote: /tmp/rpkg/rpkg-util-1-j8x35rb3/rpkg-util.spec
    Executing(%prep): /bin/sh -e /var/tmp/rpm-tmp.vE42SH
    + umask 022
    + cd /tmp/rpkg/rpkg-util-1-j8x35rb3
    + cd /tmp/rpkg/rpkg-util-1-j8x35rb3
    + rm -rf rpkg-util-2.2.git.18.5b6a884
    + /usr/bin/gzip -dc /tmp/rpkg/rpkg-util-1-j8x35rb3/rpkg-util-2.2.git.18.5b6a884.tar.gz
    + /usr/bin/tar -xof -
    + STATUS=0
    + '[' 0 -ne 0 ']'
    + cd rpkg-util-2.2.git.18.5b6a884
    + /usr/bin/chmod -Rf a+rX,u+w,g-w,o-w .
    + exit 0

For the following command, you need to have a project in COPR created. See quick tutorial at

https://docs.pagure.org/copr.copr/user_documentation.html#quick-start

on how to do it.

    $ rpkg build
    git_pack: packing directory /home/clime/rpkg-util
    git_pack: Wrote: /tmp/rpkg/rpkg-util-2-ab6awb8y/rpkg-util-2.2.git.18.0cb6974.tar.gz
    Wrote: /tmp/rpkg/rpkg-util-2-ab6awb8y/rpkg-util.spec
    Wrote: /tmp/rpkg/rpkg-util-2-ab6awb8y/rpkg-util-2.2.git.18.0cb6974-1.fc28.src.rpm
    Uploading package /tmp/rpkg/rpkg-util-2-ab6awb8y/rpkg-util-2.2.git.18.0cb6974-1.fc28.src.rpm
    100% |################################| 122kB 359kB/s eta 0:00:00
    Build was added to rpkg-util:
      https://copr.fedorainfracloud.org/coprs/build/782327/
    Created builds: 782327

Here you can see a simple operation of generation spec file from a template:

    $ rpkg spec
    Wrote: /tmp/rpkg/rpkg-util-3-1u34plfx/rpkg-util.spec

.SH "SPEC TEMPLATES"

Apart from being useful as a means to handle unpacked as well as packed content,
they also make it possible to generate certain parts of your spec file automatically
from a Git repository where your spec file is located. You need to have "spec_preprocess"
configuration option set to True in rpkg config to use this functionality.

.SS "BASIC EXAMPLE"

    Name:       {{{ git_dir_name }}}
    Version:    {{{ git_dir_version }}}
    Release:    1%{?dist}
    Summary:    This is a test package.

    License:    GPLv2+
    URL:        https://someurl.org
    VCS:        {{{ git_dir_vcs }}}

    Source:     {{{ git_dir_pack }}}

    %description
    This is a test package.

    %prep
    {{{ git_dir_setup_macro }}}

    %changelog
    {{{ git_dir_changelog }}}

Having a spec template like this in your project (instead of just a plain spec file)
allows you to build source and rpm packages automatically following git history of your project.

.SS "MACRO REFERENCE"

Please, see dedicated rpkg-macros page.

.SH USER-DEFINED MACROS

You can define your own rpkg macros that are basically just standard bash functions
read from file specified by user_macros_path config variable. Standard output of
your custom macro will be captured and the {{{ <macro_name> }}} invocation in an
input spec file will be replaced with it.

Example of a custom macro placed in rpkg.macros file:

    function git_commits_no {
        total_commits="$(git rev-list --all --count)"
        echo -n "$total_commits"
    }

.SH "SEE ALSO"
.UR "https://pagure.io/rpkg-util/"
.BR "https://pagure.io/rpkg-util/"
"""


class ManFormatter(object):
    def __init__(self, man):
        self.man = man

    def write(self, data):
        for line in data.split('\n'):
            self.man.write('  %s\n' % line)


def strip_usage(s):
    """Strip "usage: " string from beginning of string if present"""
    if s.startswith('usage: '):
        return s.replace('usage: ', '', 1)
    else:
        return s


def man_constants():
    """Global constants for man file templates"""
    today = datetime.date.today()
    today_manstr = today.strftime(r'%Y\-%m\-%d')
    return {'today': today_manstr}


def generate(parser, subparsers):
    """\
    Generate the man page on stdout

    Given the argparse based parser and subparsers arguments, generate
    the corresponding man page and write it to stdout.
    """

    # Not nice, but works: Redirect any print statement output to
    # stderr to avoid clobbering the man page output on stdout.
    man_file = sys.stdout
    sys.stdout = sys.stderr

    mf = ManFormatter(man_file)

    choices = subparsers.choices
    k = sorted(choices.keys())

    man_file.write(man_header % man_constants())

    helptext = parser.format_help()
    helptext = strip_usage(helptext)
    helptextsplit = helptext.split('\n')
    helptextsplit = [line for line in helptextsplit
                     if not line.startswith('  -h, --help')]

    man_file.write('.SS "%s"\n' % ("Global Options",))

    outflag = False
    for line in helptextsplit:
        if line == "optional arguments:":
            outflag = True
        elif line == "":
            outflag = False
        elif outflag:
            man_file.write("%s\n" % line)

    help_texts = {}
    for pa in subparsers._choices_actions:
        help_texts[pa.dest] = getattr(pa, 'help', None)

    man_file.write('.SH "COMMAND OVERVIEW"\n')

    for command in k:
        cmdparser = choices[command]
        usage = cmdparser.format_usage()
        usage = strip_usage(usage)
        usage = ''.join(usage.split('\n'))
        usage = ' '.join(usage.split())
        if help_texts[command]:
            man_file.write('.TP\n.B "%s"\n%s\n' % (usage, help_texts[command]))
        else:
            man_file.write('.TP\n.B "%s"\n' % (usage))

    man_file.write('.SH "COMMAND REFERENCE"\n')
    for command in k:
        cmdparser = choices[command]

        man_file.write('.SS "%s"\n' % cmdparser.prog)

        help = help_texts[command]
        if help and not cmdparser.description:
            if not help.endswith('.'):
                help = "%s." % help
            cmdparser.description = help

        h = cmdparser.format_help()
        mf.write(h)

    man_file.write(man_footer)


if __name__ == '__main__':
    module_path = os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))
    sys.path.insert(0, module_path)

    from rpkglib import RpkgUtil
    util = RpkgUtil()
    generate(util.parser, util.subparsers)
