import logging
import os
import re
import textwrap
import subprocess
import tempfile
import munch
import pipes
import fnmatch
import argparse

from .errors import RpkgError

try:
    from urllib.parse import urlparse
except ImportError:
    from urlparse import urlparse

log = logging.getLogger("__main__")

QUIET = 0
MODEST = 1
VERBOSE = 2
SPARTAN = 3

MACRO_HELPER_DIR = '/usr/lib/rpkg.macros.d/helpers/'
UPLOAD_EXT = ['tar', 'gz', 'bz2', 'lzma', 'xz', 'Z', 'zip', 'tff',
              'bin', 'tbz', 'tbz2', 'tgz', 'tlz', 'txz', 'pdf', 'rpm',
              'jar', 'war', 'db', 'cpio', 'jisp', 'egg', 'gem', 'spkg',
              'oxt', 'xpi', 'crate']

def changelog_entry(commit_message):
    entry = '- ' + commit_message.split('\n')[0] if commit_message else ''
    return '\n'.join(textwrap.wrap(re.sub(r'(\s+)%([^%])', r'\1%%\2', entry), 80))


def run(cmd, shell=False, env=None, cwd=None, capture=True, capture_stderr=False, throw=True):
    if capture:
        stdout = subprocess.PIPE
    else:
        stdout = None

    if capture_stderr:
        stderr = subprocess.PIPE
    else:
        stderr = None

    log.debug('Running: {0}'.format(
        cmd_repr(cmd) if not shell else cmd))

    process = subprocess.Popen(
        cmd, stdout=stdout, stderr=stderr, shell=shell, env=env, cwd=cwd)

    (out, err) = process.communicate()

    err = err.decode('utf-8').strip('\n') if err else ''

    if process.returncode != 0 and throw:
        raise RpkgError(err)

    out = out.decode('utf-8').strip('\n') if out else ''

    return munch.Munch(out=out,
                       err=err,
                       rc=process.returncode)


def edit(editor, text):
    tmp_f = tempfile.NamedTemporaryFile('w', delete=False)
    tmp_f.write(text)
    tmp_f.close()
    subprocess.check_call(editor.split()+[tmp_f.name])
    tmp_f = open(tmp_f.name, 'r')
    result = tmp_f.read()
    tmp_f.close()
    os.unlink(tmp_f.name)
    return result


def parse_config_rpmdefines(rpmdefines_str):
    return [item.strip('"\'') for item in re.findall(r'(\'[^\']*\'|"[^"]*")', rpmdefines_str)]


def parse_config_path_list(path_list_str):
    return [item.strip('"\'') for item in path_list_str.split()]


def cmd_repr(cmd):
    quoted_items = []

    for i in range(len(cmd)):
        quoted_items.append(pipes.quote(cmd[i]))

    return ' '.join(quoted_items)


def has_scheme(path):
    return urlparse(path).scheme != ''


def macro_helper_cmd(*args):
    return [MACRO_HELPER_DIR+args[0], *args[1:]]


def extract_srpm(srpm_path, destdir):
    cmd = 'rpm2cpio {0} | cpio -idm --quiet'.format(pipes.quote(srpm_path))
    run(cmd, cwd=destdir, shell=True)


def is_upload_item(item):
    for upload_ext in UPLOAD_EXT:
        if item.endswith('.'+upload_ext):
            return True
    return False


def is_reserved_item(item, script_name='rpkg'):
    for pattern in [script_name+'.conf', '.'+script_name+'*', 'sources', 'README*', '.git*']:
        if fnmatch.fnmatch(item, pattern):
            return True
    return False


class DoNothing(argparse.Action):
    def __call__(self, parser, namespace, values, option_string=None):
        pass
