import logging
import os
import rpkglib
import configparser
import getpass
import pytest
import shutil

from tempfile import NamedTemporaryFile, mkdtemp
from mock import Mock, patch, call
from munch import Munch

from . import base
from rpkglib import RpkgUtil, utils
from rpkglib.errors import RpkgError
from rpkglib.utils import run

class TestRpkgUtil(base.TestCase):

    def setUp(self):
        self.patcher = patch('sys.argv', ['rpkg', 'config'])
        self.config_mock = self.patcher.start()
        super(TestRpkgUtil, self).setUp()

    def tearDown(self):
        self.patcher.stop()
        super(TestRpkgUtil, self).tearDown()

    def test_name(self):
        rpkg_util = RpkgUtil()
        self.assertEqual(rpkg_util.name, 'rpkg')
        self.assertEqual(rpkg_util.glob_conf, '/etc/rpkg.conf')
        self.assertEqual(rpkg_util.user_conf, os.path.expanduser('~/.config/rpkg.conf'))
        self.assertEqual(rpkg_util.cwd_conf, './rpkg.conf')

        new_util = RpkgUtil('new-util')
        self.assertEqual(new_util.name, 'new-util')
        self.assertEqual(new_util.glob_conf, '/etc/new-util.conf')
        self.assertEqual(new_util.user_conf, os.path.expanduser('~/.config/new-util.conf'))
        self.assertEqual(new_util.cwd_conf, './new-util.conf')

    @patch('rpkglib.RpkgUtil.config')
    def test_logging(self, config):
        with patch('sys.argv', ['rpkg', 'config']):
            with patch('rpkglib.log') as log:
                rpkg_util = RpkgUtil()
                rpkg_util.main()
                log.assert_has_calls([call.setLevel(logging.INFO)])
                self.assertEqual(rpkg_util.args.verbosity, utils.MODEST)

        with patch('sys.argv', ['rpkg', '-v', 'config']):
            with patch('rpkglib.log') as log:
                rpkg_util = RpkgUtil()
                rpkg_util.main()
                log.assert_has_calls([call.setLevel(logging.DEBUG)])
                self.assertEqual(rpkg_util.args.verbosity, utils.VERBOSE)

        with patch('sys.argv', ['rpkg', '-q', 'config']):
            with patch('rpkglib.log') as log:
                rpkg_util = RpkgUtil()
                rpkg_util.main()
                log.assert_has_calls([call.setLevel(logging.WARNING)])
                self.assertEqual(rpkg_util.args.verbosity, utils.QUIET)

    def test_cfg(self):
        glob_conf = NamedTemporaryFile(mode='w', delete=False)
        user_conf = NamedTemporaryFile(mode='w', delete=False)
        cwd_conf = NamedTemporaryFile(mode='w', delete=False)

        parser = configparser.ConfigParser(allow_no_value=True)
        default_conf = os.path.join(rpkglib.__path__[0], 'default.conf')
        parser.read([default_conf])

        parser.set('rpkg', 'with', 'glob')
        parser.write(glob_conf)
        parser.set('rpkg', 'with', 'user')
        parser.write(user_conf)
        parser.set('rpkg', 'with', 'cwd')
        parser.write(cwd_conf)

        glob_conf.close()
        user_conf.close()
        cwd_conf.close()

        rpkg_util = RpkgUtil()
        rpkg_util.glob_conf = glob_conf.name
        rpkg_util.user_conf = '/dev/null'
        rpkg_util.cwd_conf = '/dev/null'
        cfg = rpkg_util.cfg
        self.assertEqual(cfg.rpkg.bcond_with, 'glob')

        rpkg_util = RpkgUtil()
        rpkg_util.glob_conf = glob_conf.name
        rpkg_util.user_conf = user_conf.name
        rpkg_util.cwd_conf = '/dev/null'
        cfg = rpkg_util.cfg
        self.assertEqual(cfg.rpkg.bcond_with, 'user')

        rpkg_util = RpkgUtil()
        rpkg_util.glob_conf = glob_conf.name
        rpkg_util.user_conf = user_conf.name
        rpkg_util.cwd_conf = cwd_conf.name
        cfg = rpkg_util.cfg
        self.assertEqual(cfg.rpkg.bcond_with, 'cwd')

        self.init_repo()
        repo_conf = open(os.path.join(self.cwd, 'rpkg.conf'), mode='w')
        parser.set('rpkg', 'with', 'repo')
        parser.write(repo_conf)
        repo_conf.close()

        os.mkdir('./subdir')
        os.chdir('./subdir')

        rpkg_util = RpkgUtil()
        rpkg_util.glob_conf = glob_conf.name
        rpkg_util.user_conf = user_conf.name
        rpkg_util.repo_conf = repo_conf.name
        rpkg_util.cwd_conf = cwd_conf.name
        cfg = rpkg_util.cfg
        self.assertEqual(cfg.rpkg.bcond_with, 'cwd')

        rpkg_util = RpkgUtil()
        rpkg_util.glob_conf = glob_conf.name
        rpkg_util.user_conf = user_conf.name
        rpkg_util.repo_conf = repo_conf.name
        rpkg_util.cwd_conf = '/dev/null'
        cfg = rpkg_util.cfg
        self.assertEqual(cfg.rpkg.bcond_with, 'repo')

        head = self.repo_head()
        head_short = self.repo_head_short()
        self.assertEqual(cfg.git_props.branch, 'master')
        self.assertEqual(cfg.git_props.remote, 'origin')
        self.assertEqual(cfg.git_props.remote_netloc, 'pagure.io')
        self.assertEqual(cfg.git_props.remote_url, 'https://pagure.io/rpkg-util')
        self.assertEqual(cfg.git_props.root, self.cwd)
        self.assertEqual(cfg.git_props.head, head)
        self.assertEqual(cfg.git_props.head_short, head_short)
        self.assertEqual(cfg.git_props.status, '')
        self.assertEqual(cfg.git_props.merged_tag_refs, '')
        self.assertEqual(cfg.git_props.submodule_refs, '{} {}'.format(head, self.cwd))
        self.assertEqual(cfg.git_props.dirty, '')

        self.assertEqual(cfg.DEFAULT.path, os.path.join(self.cwd, 'subdir'))
        self.assertEqual(cfg.DEFAULT.user, getpass.getuser())

        override_conf = NamedTemporaryFile(mode='w', delete=False)
        parser.set('rpkg', 'with', '')
        parser.set('rpkg', 'without', 'override')
        parser.write(override_conf)
        override_conf.close()
        with patch('sys.argv', ['rpkg', '-f', override_conf.name, 'config']):
            rpkg_util = RpkgUtil()
            rpkg_util.glob_conf = glob_conf.name
            rpkg_util.user_conf = user_conf.name
            rpkg_util.repo_conf = repo_conf.name
            rpkg_util.cwd_conf = cwd_conf.name
            cfg = rpkg_util.cfg
            self.assertEqual(cfg.rpkg.bcond_with, '')
            self.assertEqual(cfg.rpkg.bcond_without, 'override')

        with patch('sys.argv', ['rpkg', '-f', override_conf.name, '-c',
                                'rpkg.without=cheese', 'config']):
            rpkg_util = RpkgUtil()
            rpkg_util.glob_conf = glob_conf.name
            rpkg_util.user_conf = user_conf.name
            rpkg_util.repo_conf = repo_conf.name
            rpkg_util.cwd_conf = cwd_conf.name
            cfg = rpkg_util.cfg
            self.assertEqual(cfg.rpkg.bcond_without, 'cheese')

        with patch('sys.argv', ['rpkg', '-c', 'rpkg.with=cheese', 'config']):
            rpkg_util = RpkgUtil()
            rpkg_util.glob_conf = glob_conf.name
            rpkg_util.user_conf = user_conf.name
            rpkg_util.repo_conf = repo_conf.name
            rpkg_util.cwd_conf = cwd_conf.name
            cfg = rpkg_util.cfg
            self.assertEqual(cfg.rpkg.bcond_with, 'cheese')

        os.unlink(glob_conf.name)
        os.unlink(user_conf.name)
        os.unlink(cwd_conf.name)
        os.unlink(repo_conf.name)
        os.unlink(override_conf.name)

    @patch('rpkglib.open')
    @patch('rpkglib.RpmPackage')
    @patch('rpkglib.GitClient')
    def test_rpm_package(self, git_client_cls, rpm_package_cls, open_mock):
        git_props = Munch(
            root='/', branch='',
            remote='', remote_url='',
            remote_netloc='', head='', status='',
            head_short='', merged_tag_refs='',
            submodule_refs='', dirty='')

        git_client_cls.git_props = lambda: git_props
        git_client_mock = Mock()
        git_client_cls.return_value = git_client_mock

        rpkg_util = RpkgUtil()
        rpm_package_cls.locate_spec.return_value = 'foo.spec.rpkg'
        rpm_package = rpkg_util.rpm_package
        rpm_package_cls.locate_spec.assert_called_with(self.cwd)
        rpm_package_cls.assert_called()
        git_client_mock.assert_not_called()
        open_mock.assert_called_with('foo.spec.rpkg', 'r')

        git_client_mock.reset_mock()
        open_mock.reset_mock()

        git_props.head = 'head'
        rpkg_util = RpkgUtil()
        rpm_package = rpkg_util.rpm_package
        git_client_mock.read_file.assert_called_with('head', 'foo.spec.rpkg')
        open_mock.assert_not_called()

        git_client_mock.reset_mock()
        open_mock.reset_mock()

        git_props.head = 'head'
        git_props.dirty = 'yes'
        rpkg_util = RpkgUtil()
        rpm_package = rpkg_util.rpm_package
        git_client_mock.read_file.assert_not_called()

        git_client_mock.reset_mock()
        open_mock.reset_mock()

        git_props.head = 'head'
        git_props.dirty = ''
        git_client_mock.read_file.side_effect = RpkgError
        rpkg_util = RpkgUtil()
        rpm_package = rpkg_util.rpm_package
        git_client_mock.read_file.assert_called()
        open_mock.assert_called_with('foo.spec.rpkg', 'r')

        git_client_mock.reset_mock()
        open_mock.reset_mock()

        git_props.head = ''
        git_props.dirty = ''
        with patch('sys.argv', ['rpkg', 'spec', '--spec', 'some_path']):
            rpkg_util = RpkgUtil()
            rpm_package = rpkg_util.rpm_package
            open_mock.assert_called_with('some_path', 'r')

    @patch('rpkglib.GitClient')
    def test_git_client(self, git_client_cls):
        rpkg_util = RpkgUtil()
        git_client_cls.git_props = lambda: Munch(
            root='/', branch='',
            remote='', remote_url='',
            remote_netloc='', head='', status='',
            head_short='', merged_tag_refs='',
            submodule_refs='', dirty='')
        git_client = rpkg_util.git_client
        git_client_cls.assert_called()

    @patch('rpkglib.LookasideCache')
    def test_lookaside_cls(self, lookaside_cls):
        rpkg_util = RpkgUtil()
        lookaside = rpkg_util.lookaside
        lookaside_cls.assert_called()

    @patch('rpkglib.RpkgUtil.git_client')
    def test_add(self, git_client):
        with patch('sys.argv', ['rpkg', 'add']):
            RpkgUtil().main()
            git_client.add.assert_called()

    @patch('rpkglib.RpkgUtil.cfg_parser')
    def test_config(self, cfg_parser):
        with patch('sys.argv', ['rpkg', 'config']):
            cfg_parser.write = Mock()
            RpkgUtil().main()
            cfg_parser.write.assert_called()

    @patch('rpkglib.RpkgUtil.git_client')
    def test_clean(self, git_client):
        with patch('sys.argv', ['rpkg', 'clean']):
            RpkgUtil().main()
            git_client.clean.assert_called()

    @patch('builtins.print')
    @patch('rpkglib.RpkgUtil.rpm_package')
    def test_clog(self, rpm_package, print_mock):
        with patch('sys.argv', ['rpkg', 'clog']):
            RpkgUtil().main()
            rpm_package.clog.assert_called()
            print_mock.assert_called()

    @patch('rpkglib.RpkgUtil.git_client')
    def test_clone(self, git_client):
        with patch('sys.argv', ['rpkg', 'clone', 'https://pagure.io/rpkg-util']):
            RpkgUtil().main()
            git_client.clone.assert_called()
            git_client.clone_with_dirs.assert_not_called()

        git_client.reset_mock()
        with patch('sys.argv', ['rpkg', 'clone', '-B', 'https://pagure.io/rpkg-util']):
            RpkgUtil().main()
            git_client.clone.assert_not_called()
            git_client.clone_with_dirs.assert_called()

    @patch('rpkglib.RpkgUtil.rpm_package')
    @patch('rpkglib.RpkgUtil.git_client')
    def test_commit(self, git_client, rpm_package):
        with patch('sys.argv', ['rpkg', 'commit', '-m', 'foo']):
            RpkgUtil().main()
            git_client.commit.assert_called_with(False, False, ['foo'], None, False, [])

        with patch('sys.argv', ['rpkg', 'commit', '--amend']):
            RpkgUtil().main()
            git_client.commit.assert_called_with(False, True, [], None, False, [])

        git_client.reset_mock()
        rpm_package.clog.return_value = '- hello world'
        with patch('sys.argv', ['rpkg', 'commit', '-m', 'foo', '--with-changelog']):
            RpkgUtil().main()
            git_client.commit.assert_called_with(False, False, ['foo', '- hello world'], None, False, [])

        git_client.reset_mock()
        rpm_package.clog.return_value = 'hello world'
        with patch('sys.argv', ['rpkg', 'commit', '-F', 'msgfile', '-s', 'somefile']):
            RpkgUtil().main()
            git_client.commit.assert_called_with(False, False, [], 'msgfile', True, ['somefile'])

        git_client.reset_mock()
        with patch('sys.argv', ['rpkg', 'commit', '-a' , '-t', '-p']):
            RpkgUtil().main()
            git_client.commit.assert_called_with(True, False, [], None, False, [])
            rpm_package.nvrid.assert_called()
            git_client.add_tag.assert_called()
            git_client.push.assert_called()

    @patch('rpkglib.RpkgUtil.git_client')
    @patch('rpkglib.RpkgUtil.lookaside')
    @patch('rpkglib.RpkgUtil.rpm_package')
    def test_compile(self, rpm_package, lookaside, git_client):
        with patch('sys.argv', ['rpkg', 'compile']):
            RpkgUtil().main()
            git_client.repo_path.assert_called()
            lookaside.sources.assert_called()
            rpm_package.compile.assert_called()

    @patch('rpkglib.RpkgUtil.git_client')
    def test_diff(self, git_client):
        with patch('sys.argv', ['rpkg', 'diff']):
            RpkgUtil().main()
            git_client.diff.assert_called()

    @patch('rpkglib.RpkgUtil.rpm_package')
    @patch('rpkglib.print')
    def test_gimmespec(self, print_mock, rpm_package):
        rpm_package.spec_path = 'somepath'
        with patch('sys.argv', ['rpkg', 'gimmespec']):
            RpkgUtil().main()
            print_mock.assert_called_with('somepath')

    @patch('rpkglib.GitClient.show_diff_index')
    @patch('rpkglib.extract_srpm')
    @patch('rpkglib.tempfile')
    @patch('rpkglib.log', Mock())
    @patch('rpkglib.lookaside_cache.log', Mock())
    def test_import_srpm(self, tempfile, extract_srpm, show_diff_index):
        with patch('rpkglib.RpkgUtil.git_props') as git_props:
            git_props = Munch(dirty='yes')
            with patch('sys.argv', ['rpkg', '-v', 'import', 'srpm_path']):
                with pytest.raises(RpkgError):
                    RpkgUtil().main()

        self.init_repo()
        run('echo 1 > foo.spec', shell=True)
        run('echo 1 > bar1.tar.gz', shell=True)
        run('echo 1 > baz1.patch', shell=True)
        run('touch README', shell=True)
        run('mkdir subdir', shell=True)
        run('touch subdir/foo', shell=True)
        run('echo "SHA256 (bar1.tar.gz) = 4355a46b19d348dc2f57c046f8ef63d4538ebb936000f3c9ee954a27460dd865" > sources', shell=True)
        run('git add -A && git commit -m "new commit"', shell=True)

        tmpdir = mkdtemp()
        tempfile.mkdtemp.return_value = tmpdir
        run('echo 2 > foo.spec', shell=True, cwd=tmpdir)
        run('echo 2 > bar2.tar.gz', shell=True, cwd=tmpdir)
        run('echo 2 > baz2.patch', shell=True, cwd=tmpdir)
        run('echo 2 > bazz2.patch', shell=True, cwd=tmpdir)

        with patch('sys.argv', ['rpkg', 'import', 'srpm_path', '--offline', '--skip-diffs']):
            RpkgUtil().main()

        items = os.listdir()
        self.assertEqual(items, ['.gitignore', 'foo.spec', 'bar2.tar.gz',
                                  'baz2.patch', 'bazz2.patch', 'sources',
                                  'subdir', 'README', '.git'])
        items = os.listdir('subdir')
        self.assertEqual(items, ['foo'])
        content = open('foo.spec', 'r').read()
        self.assertEqual(content, '2\n')
        content = open('bar2.tar.gz', 'r').read()
        self.assertEqual(content, '2\n')
        content = open('baz2.patch', 'r').read()
        self.assertEqual(content, '2\n')
        content = open('bazz2.patch', 'r').read()
        self.assertEqual(content, '2\n')

        sources_content = open('sources', 'r').read()
        self.assertEqual(sources_content, 'SHA512 (bar2.tar.gz) = 63e22ec2fbeebabf005e58fbfb0eee607c4aa417045a68a0cc63767b048e3559268d35e72f367d3b2dbd5dbddf12fc4397762ba149260b3795a0391713bddcd7\n')
        gitignore_content = open('.gitignore', 'r').read()
        self.assertEqual(gitignore_content, '/bar2.tar.gz\n')

        status = run('git status --porcelain', shell=True).out
        self.assertEqual(status, 'A  .gitignore\nD  bar1.tar.gz\nD  baz1.patch\nA  baz2.patch\nA  bazz2.patch\nM  foo.spec\nM  sources')
        show_diff_index.assert_not_called()

        tmpdir = mkdtemp()
        tempfile.mkdtemp.return_value = tmpdir
        run('git reset --hard', shell=True)
        with patch('sys.argv', ['rpkg', 'import', 'srpm_path', '--offline']):
            RpkgUtil().main()
            show_diff_index.assert_called()

    @patch('rpkglib.RpkgUtil.git_client')
    @patch('rpkglib.RpkgUtil.lookaside')
    @patch('rpkglib.RpkgUtil.rpm_package')
    def test_install(self, rpm_package, lookaside, git_client):
        with patch('sys.argv', ['rpkg', 'install']):
            RpkgUtil().main()
            git_client.repo_path.assert_called()
            lookaside.sources.assert_called()
            rpm_package.install.assert_called()

    @patch('rpkglib.RpkgUtil.rpm_package')
    def test_lint(self, rpm_package):
        with patch('sys.argv', ['rpkg', 'lint']):
            RpkgUtil().main()
            rpm_package.lint.assert_called()
            self.assertTrue(rpm_package.use_existing_outdir)

    @patch('rpkglib.RpkgUtil.git_client')
    @patch('rpkglib.RpkgUtil.lookaside')
    @patch('rpkglib.RpkgUtil.rpm_package')
    def test_local(self, rpm_package, lookaside, git_client):
        with patch('sys.argv', ['rpkg', 'local']):
            RpkgUtil().main()
            git_client.repo_path.assert_called()
            lookaside.sources.assert_called()
            rpm_package.local.assert_called()

    @patch('rpkglib.RpkgUtil.git_client')
    def test_log(self, git_client):
        with patch('sys.argv', ['rpkg', 'log']):
            RpkgUtil().main()
            git_client.log.assert_called()

    @patch('rpkglib.RpkgUtil.git_client')
    def test_merge(self, git_client):
        with patch('sys.argv', ['rpkg', 'merge']):
            RpkgUtil().main()
            git_client.merge.assert_called()

        git_client.reset_mock()
        with patch('sys.argv', ['rpkg', 'merge', '--continue']):
            RpkgUtil().main()
            git_client.merge_continue.assert_called()

        git_client.reset_mock()
        with patch('sys.argv', ['rpkg', 'merge', '--abort']):
            RpkgUtil().main()
            git_client.merge_abort.assert_called()

    @patch('rpkglib.RpkgUtil.git_client')
    @patch('rpkglib.RpkgUtil.rpm_package')
    def test_new(self, rpm_package, git_client):
        with patch('sys.argv', ['rpkg', 'new']):
            RpkgUtil().main()
            rpm_package.nvre.assert_called()
            git_client.new.assert_called()

    @patch('rpkglib.RpkgUtil.git_client')
    @patch('rpkglib.RpkgUtil.lookaside')
    @patch('rpkglib.RpkgUtil.rpm_package')
    def test_prep(self, rpm_package, lookaside, git_client):
        with patch('sys.argv', ['rpkg', 'prep']):
            RpkgUtil().main()
            git_client.repo_path.assert_called()
            lookaside.sources.assert_called()
            rpm_package.prep.assert_called()

    @patch('rpkglib.RpkgUtil.git_client')
    def test_pull(self, git_client):
        with patch('sys.argv', ['rpkg', 'pull']):
            RpkgUtil().main()
            git_client.pull.assert_called()

    @patch('rpkglib.RpkgUtil.git_client')
    def test_push(self, git_client):
        with patch('sys.argv', ['rpkg', 'push']):
            RpkgUtil().main()
            git_client.push.assert_called()

    @patch('rpkglib.RpkgUtil.git_client')
    def test_reset(self, git_client):
        with patch('sys.argv', ['rpkg', 'reset']):
            RpkgUtil().main()
            git_client.reset.assert_called()

        with patch('sys.argv', ['rpkg', 'reset', '--mixed', '--', 'commit']):
            RpkgUtil().main()
            git_client.reset.assert_called_with(1, 'commit')

    @patch('rpkglib.RpkgUtil.git_client')
    @patch('rpkglib.RpkgUtil.lookaside')
    def test_sources(self, lookaside, git_client):
        with patch('sys.argv', ['rpkg', 'sources']):
            git_client.repo_path.return_value = 'repo_path'
            RpkgUtil().main()
            git_client.repo_path.assert_called()
            lookaside.sources.assert_called_with('repo_path', None)

        lookaside.reset_mock()
        git_client.reset_mock()
        with patch('sys.argv', ['rpkg', 'sources',
                                '--outdir', 'outdir', '--repo-path', 'repo_path']):
            RpkgUtil().main()
            git_client.repo_path.assert_not_called()
            lookaside.sources.assert_called_with('repo_path', 'outdir')

    @patch('rpkglib.RpkgUtil.rpm_package')
    @patch('rpkglib.print')
    def test_spec(self, print, rpm_package):
        with patch('sys.argv', ['rpkg', 'spec', '--sources']):
            RpkgUtil().main()
            rpm_package.final_spec.assert_called()
            self.assertTrue(rpm_package.produce_sources)

        rpm_package.reset_mock()
        with patch('sys.argv', ['rpkg', 'spec']):
            RpkgUtil().main()
            rpm_package.final_spec.assert_called()
            self.assertFalse(rpm_package.produce_sources)

        rpm_package.reset_mock()
        with patch('sys.argv', ['rpkg', 'spec', '--print']):
            RpkgUtil().main()
            rpm_package.final_spec.assert_not_called()
            rpm_package.final_spec_text.assert_called()
            print.assert_called()
            self.assertFalse(rpm_package.produce_sources)

    @patch('rpkglib.RpkgUtil.git_client')
    @patch('rpkglib.RpkgUtil.lookaside')
    @patch('rpkglib.RpkgUtil.rpm_package')
    def test_srpm(self, rpm_package, lookaside, git_client):
        with patch('sys.argv', ['rpkg', 'srpm']):
            RpkgUtil().main()
            git_client.repo_path.assert_called()
            lookaside.sources.assert_called()
            rpm_package.srpm.assert_called()

    @patch('rpkglib.RpkgUtil.git_client')
    def test_status(self, git_client):
        with patch('sys.argv', ['rpkg', 'status']):
            RpkgUtil().main()
            git_client.status.assert_called()

    @patch('rpkglib.RpkgUtil.git_client')
    def test_switch(self, git_client):
        with patch('sys.argv', ['rpkg', 'switch']):
            RpkgUtil().main()
            git_client.list_branches.assert_called()

        git_client.reset_mock()
        with patch('sys.argv', ['rpkg', 'switch', '-l']):
            RpkgUtil().main()
            git_client.list_branches.assert_called()

        git_client.reset_mock()
        with patch('sys.argv', ['rpkg', 'switch', '--fetch']):
            RpkgUtil().main()
            git_client.fetch.assert_called()
            git_client.list_branches.assert_called()

        git_client.reset_mock()
        with patch('sys.argv', ['rpkg', 'switch', '--fetch', 'target']):
            RpkgUtil().main()
            git_client.fetch.assert_called()
            git_client.checkout.assert_called()
            git_client.list_branches.assert_not_called()

    @patch('rpkglib.RpkgUtil.git_client')
    @patch('rpkglib.RpkgUtil.rpm_package')
    def test_tag(self, rpm_package, git_client):
        with patch('sys.argv', ['rpkg', 'tag']):
            RpkgUtil().main()
            rpm_package.nvrid.assert_called()
            git_client.add_tag.assert_called()

        rpm_package.reset_mock()
        with patch('sys.argv', ['rpkg', 'tag', 'foo']):
            RpkgUtil().main()
            rpm_package.nvrid.assert_not_called()
            git_client.add_tag.assert_called()

    @patch('rpkglib.RpkgUtil.rpm_package')
    def test_unused_patches(self, rpm_package):
        with patch('sys.argv', ['rpkg', 'unused-patches']):
            RpkgUtil().main()
            rpm_package.unused_patches.assert_called()

    @patch('rpkglib.log')
    @patch('rpkglib.RpkgUtil.git_client')
    @patch('rpkglib.RpkgUtil.lookaside')
    def test_upload(self, lookaside, git_client, log):
        with patch('sys.argv', ['rpkg', 'upload', 'file1', 'file2']):
            source_entries = [
                Munch(filename='filename',
                      hashtype='sha512',
                      hash='hash')]
            lookaside.upload.return_value = source_entries
            git_client.repo_path.return_value = 'repo_path'
            rpkg_util = RpkgUtil()
            rpkg_util.git_props = Munch(root='')
            rpkg_util.main()
            git_client.repo_path.assert_called()
            lookaside.upload.assert_called_with('repo_path', ['file1', 'file2'], offline=False)
            git_client.write_ignores.assert_not_called()
            git_client.add_to_index.assert_not_called()
            git_client.status.assert_not_called()
            lookaside.write_source_entries.assert_called_with(
                source_entries, append=True)

        git_client.reset_mock()
        lookaside.reset_mock()
        with patch('sys.argv', ['rpkg', 'upload', '--repo-path', 'repo_path2', 'file1', 'file2', '--offline']):
            rpkg_util = RpkgUtil()
            rpkg_util.main()
            git_client.repo_path.assert_not_called()
            lookaside.upload.assert_called_with('repo_path2', ['file1', 'file2'], offline=True)

        git_client.reset_mock()
        lookaside.reset_mock()
        with patch('sys.argv', ['rpkg', 'upload', 'file1', 'file2']):
            git_client.repo_path.return_value = 'repo_path'
            rpkg_util = RpkgUtil()
            rpkg_util.git_props = Munch(root='/root')
            rpkg_util.main()
            lookaside.upload.assert_called_with('repo_path', ['file1', 'file2'], offline=False)
            git_client.write_ignores.assert_called()
            git_client.add_to_index.assert_called()
            git_client.status.assert_called()

    @patch('rpkglib.RpkgUtil.rpm_package')
    @patch('rpkglib.print')
    def test_verify_files(self, print_mock, rpm_package):
        with patch('sys.argv', ['rpkg', 'nvr']):
            RpkgUtil().main()
            rpm_package.nvrid.assert_called()
            print_mock.assert_called()
