import os
import pytest
import shutil

from munch import Munch
from tempfile import mkdtemp, mkstemp
from mock import patch, Mock

from . import base
from .spec_templates import SPEC_TEMPLATE, SPEC_WITH_PATCH_TEMPLATE,\
    INVALID_SPEC_TEMPLATE, SPEC_TEMPLATE_WITH_EPOCH, SPEC_TEMPLATE_WITH_CLOG,\
    SPEC_TEMPLATE_WITH_PATCH

from rpkglib.errors import RpkgError
from rpkglib import RpmPackage, utils


class TestRpmPackage(base.TestCase):

    def setUp(self):
        super(TestRpmPackage, self).setUp()

    def tearDown(self):
        super(TestRpmPackage, self).tearDown()

    def test_locate_spec(self):
        rpm_package = RpmPackage(self.cwd)

        spec_path = os.path.join(self.cwd, 'x.spec')
        open(spec_path, 'w').close()

        result = rpm_package.locate_spec(self.cwd)
        self.assertEqual(result, spec_path)
        os.unlink(spec_path)

        spec_rpkg_path = os.path.join(self.cwd, 'x.spec.rpkg')
        open(spec_rpkg_path, 'w').close()

        result = rpm_package.locate_spec(self.cwd)
        self.assertEqual(result, spec_rpkg_path)
        os.unlink(spec_rpkg_path)

        spec_hidden_path = os.path.join(self.cwd, '.x.spec')
        open(spec_hidden_path, 'w').close()

        with pytest.raises(RpkgError):
            result = rpm_package.locate_spec(self.cwd)

        os.unlink(spec_hidden_path)

    def test_create_outdir(self):
        spec_path = os.path.join(self.cwd, 'x.spec')
        open(spec_path, 'w').close()

        tmpdir = mkdtemp()
        rpm_package = RpmPackage(self.cwd, base_output_path=tmpdir,
                                 spec_path=spec_path)
        rpm_package.create_outdir()

        result = os.listdir(tmpdir)
        self.assertEqual(len(result), 1)
        self.assertTrue('x-1-' in result[0])

        rpm_package.create_outdir()

        result = sorted(os.listdir(tmpdir))
        self.assertEqual(len(result), 2)
        self.assertTrue('x-2-' in result[1])

        shutil.rmtree(tmpdir)
        rpm_package = RpmPackage(self.cwd, base_output_path=tmpdir,
                                 spec_path=spec_path)
        rpm_package.create_outdir()

        result = os.listdir(tmpdir)
        self.assertEqual(len(result), 1)
        self.assertTrue('x-1-' in result[0])

        rpm_package = RpmPackage(self.cwd, base_output_path=tmpdir,
                                 spec_path=spec_path, use_existing_outdir=True)
        result = rpm_package.create_outdir()
        self.assertTrue('x-1-' in result)

        result = os.listdir(tmpdir)
        self.assertEqual(len(result), 1)
        self.assertTrue('x-1-' in result[0])

        shutil.rmtree(tmpdir)

    def test_rpm_opts(self):
        rpm_package = RpmPackage(self.cwd)
        rpm_package = RpmPackage(self.cwd, rpmdefines=['dist xxx'],
                                 bump_rpmdefines=['dist %nil'])
        result = rpm_package.rpm_opts()
        self.assertEqual(result, ['--define', 'dist xxx'])

        rpm_package = RpmPackage(self.cwd, rpmdefines=['dist xxx'],
                                 bump_rpmdefines=['dist %nil'], version_bump=True)
        result = rpm_package.rpm_opts()
        self.assertEqual(result, ['--define', 'dist %nil'])

        rpm_package = RpmPackage(self.cwd, rpmdefines=['dist xxx'],
                                 bump_rpmdefines=['dist %nil'], release_bump=True)
        result = rpm_package.rpm_opts()
        self.assertEqual(result, ['--define', 'dist %nil'])

    @patch('rpkglib.RpmPackage.outdir', 'outdir')
    @patch('rpkglib.RpmPackage.rpm_opts')
    def test_rpmbuild_opts(self, rpm_opts):
        expected_base = [
            '--define', '_sourcedir outdir',
            '--define', '_specdir outdir',
            '--define', '_builddir outdir',
            '--define', '_buildrootdir outdir',
            '--define', '_srcrpmdir outdir',
            '--define', '_rpmdir outdir',
        ]
        rpm_opts.return_value = ['--define', 'dist %nil']

        rpm_package = RpmPackage(self.cwd)
        result = rpm_package.rpmbuild_opts()
        self.assertEqual(result, expected_base+['--define', 'dist %nil'])

        rpm_package = RpmPackage(self.cwd, bcond_with=['cheese'])
        result = rpm_package.rpmbuild_opts()
        self.assertEqual(result, expected_base+['--with', 'cheese',
                                                 '--define', 'dist %nil'])

        rpm_package = RpmPackage(self.cwd,
                                 bcond_with=['cheese'],
                                 bcond_without=['olives', 'anchovies'])
        result = rpm_package.rpmbuild_opts()
        self.assertEqual(result, expected_base+['--with', 'cheese',
                                                 '--without', 'olives',
                                                 '--without', 'anchovies',
                                                 '--define', 'dist %nil'])

    @patch('rpkglib.RpmPackage.nvre')
    def test_nvrid(self, nvre):
        nvre.return_value = Munch({
            'name': 'foo',
            'version': '1',
            'release': '1',
            'epoch': '1',
        })
        rpm_package = RpmPackage(self.cwd)
        result = rpm_package.nvrid()
        self.assertEqual(result, 'foo-1-1')

        result = rpm_package.nvrid(name='bar')
        self.assertEqual(result, 'bar-1-1')

        result = rpm_package.nvrid(
            name='bar', version='2', release='2')
        self.assertEqual(result, 'bar-2-2')

    @patch('rpkglib.RpmPackage.final_tmp_spec')
    def test_nvre(self, final_tmp_spec):
        rpm_package = RpmPackage(self.cwd, outdir='/tmp/outdir')

        spec_path = self.dump_spec(
            SPEC_TEMPLATE, name='testpkg',
            version='1', release='1', epoch=None)
        final_tmp_spec.return_value = spec_path
        result = rpm_package.nvre()
        self.assertEqual(result, Munch({'name': 'testpkg', 'version': '1',
                                         'release': '1', 'epoch': '(none)'}))

        spec_path = self.dump_spec(
            SPEC_TEMPLATE_WITH_EPOCH, name='testpkg2',
            version='1', release='1', epoch='2')
        final_tmp_spec.return_value = spec_path
        result = rpm_package.nvre()
        self.assertEqual(result, Munch({'name': 'testpkg2', 'version': '1',
                                         'release': '1', 'epoch': '2'}))

    def test_preproc_source_params(self):
        rpm_package = RpmPackage(self.cwd, default_macros=['git.bash'])
        result = rpm_package.preproc_source_params()
        self.assertEqual(result, '-s /usr/lib/rpkg.macros.d/git.bash')

        with pytest.raises(RpkgError):
            rpm_package = RpmPackage(self.cwd,
                                     default_macros=['git.bash'],
                                     user_macros=['/foo/bar.bash'])
            rpm_package.preproc_source_params()

        _, user_macros_file = mkstemp()
        rpm_package = RpmPackage(self.cwd,
                                 default_macros=['git.bash'],
                                 user_macros=[user_macros_file])
        result = rpm_package.preproc_source_params()
        self.assertEqual(result, '-s /usr/lib/rpkg.macros.d/git.bash'
                          ' -s {0}'.format(user_macros_file))

        os.unlink(user_macros_file)

    def test_preproc_env_params(self):
        outdir = mkdtemp()
        rpm_package = RpmPackage(self.cwd, outdir=outdir)
        spec_file = os.path.join(self.cwd, 'x.spec.rpkg')

        open(spec_file, 'w').close()
        result = rpm_package.preproc_env_params(spec_file)
        self.assertEqual(result, '-e INPUT_PATH={0} -e INPUT_DIR_PATH={1}'
                          ' -e OUTDIR={2}'.format(spec_file, self.cwd, outdir))

        rpm_package = RpmPackage(self.cwd, outdir=outdir,
                                 produce_sources=False)
        result = rpm_package.preproc_env_params(spec_file)
        self.assertEqual(result, '-e INPUT_PATH={0} -e INPUT_DIR_PATH={1}'
                          .format(spec_file, self.cwd, outdir))

        rpm_package = RpmPackage(self.cwd, outdir=outdir,
                                 produce_sources=False,
                                 version_bump=True,
                                 release_bump=True)
        result = rpm_package.preproc_env_params(spec_file)
        self.assertEqual(result, '-e INPUT_PATH={0} -e INPUT_DIR_PATH={1}'
                          ' -e VERSION_BUMP=1 -e RELEASE_BUMP=1'
                          .format(spec_file, self.cwd, outdir))

        rpm_package = RpmPackage(self.cwd, outdir=outdir,
                                 produce_sources=False,
                                 verbosity=2)
        result = rpm_package.preproc_env_params(spec_file)
        self.assertEqual(result, '-e INPUT_PATH={0} -e INPUT_DIR_PATH={1}'
                          ' -e VERBOSE=1'.format(spec_file, self.cwd, outdir))

        shutil.rmtree(outdir)

    def test_preproc_git_props_params(self):
        git_props = Munch({
            'root': '/tmp/root',
            'branch': 'master',
            'remote': 'origin',
            'remote_url': 'https://pagure.io/rpkg-util',
            'remote_netloc': 'pagure.io',
            'head': 'somesha',
            'head_short': 'shortsha',
            'status': ' M file',
            'merged_tag_refs': 'somesha tag-1-2',
            'submodule_refs': 'somesha /tmp/root',
            'dirty': 'yes',
        })
        rpm_package = RpmPackage(self.cwd, git_props=git_props)

        tmpfile1 = Mock()
        tmpfile1.name = 'status_file'
        tmpfile2 = Mock()
        tmpfile2.name = 'merged_tag_refs_file'
        tmpfile3 = Mock()
        tmpfile3.name = 'submodule_refs_file'

        result = rpm_package.preproc_git_props_params(tmpfile1, tmpfile2, tmpfile3)
        self.assertEqual(result, '-e GIT_ROOT=/tmp/root -e GIT_BRANCH=master -e GIT_REMOTE=origin '
                          '-e GIT_REMOTE_URL=https://pagure.io/rpkg-util -e GIT_REMOTE_NETLOC=pagure.io '
                          '-e GIT_HEAD=somesha -e GIT_HEAD_SHORT=shortsha -e GIT_STATUS_FILE=status_file '
                          '-e GIT_MERGED_TAG_REFS_FILE=merged_tag_refs_file -e GIT_SUBMODULE_REFS_FILE=submodule_refs_file '
                          '-e GIT_DIRTY=yes')

        tmpfile1.write.assert_called_with(' M file\n')
        tmpfile2.write.assert_called_with('somesha tag-1-2\n')
        tmpfile3.write.assert_called_with('somesha /tmp/root\n')

        tmpfile1.close.assert_called()
        tmpfile2.close.assert_called()
        tmpfile3.close.assert_called()

    @patch('rpkglib.rpm_package.os.unlink')
    @patch('rpkglib.rpm_package.run')
    @patch('rpkglib.rpm_package.log')
    @patch('rpkglib.rpm_package.NamedTemporaryFile')
    @patch('rpkglib.RpmPackage.preproc_env_params')
    @patch('rpkglib.RpmPackage.preproc_source_params')
    @patch('rpkglib.RpmPackage.preproc_git_props_params')
    def test_final_spec_text(self, preproc_git_props_params, preproc_source_params,
                             preproc_env_params, tmpfile, log, run, unlink):
        spec_path = os.path.join(self.cwd, 'x.spec.rpkg')
        with open(spec_path, 'w') as spec_file:
            spec_file.write('content')

        preproc_source_params.return_value = '-s /tmp/foo.bash'
        preproc_env_params.return_value = '-e INPUT_PATH {0}'.format(spec_path)
        preproc_git_props_params.return_value = '-e GIT_ROOT {0}'.format(self.cwd)

        tmpfile_mock = Mock()
        tmpfile_mock.name = '/tmp/file'
        tmpfile.return_value = tmpfile_mock

        rpm_package = RpmPackage(self.cwd)
        run.return_value = Munch(out='rendered_content', err=None, rc=None)
        result = rpm_package.final_spec_text()
        run.assert_called_with('cat {0} | preproc -C {1} -s /tmp/foo.bash -e INPUT_PATH {2} -e GIT_ROOT {1}'
                               .format('/tmp/file', self.cwd, spec_path), capture=True, shell=True)
        preproc_env_params.assert_called_with(spec_path)
        self.assertEqual(result, 'rendered_content')
        tmpfile_mock.write.assert_called_with('content')
        self.assertEqual(tmpfile.call_count, 4)
        self.assertEqual(unlink.call_count, 4)

        run.reset_mock()
        unlink.reset_mock()
        tmpfile.reset_mock()
        preproc_env_params.reset_mock()

        preproc_env_params.return_value = '-e INPUT_PATH /tmp/y.spec.rpkg'
        rpm_package = RpmPackage(self.cwd, 'content2', '/tmp/y.spec.rpkg')
        result = rpm_package.final_spec_text()
        run.assert_called_with('cat {0} | preproc -C {1} -s /tmp/foo.bash -e INPUT_PATH /tmp/y.spec.rpkg -e GIT_ROOT {1}'
                               .format('/tmp/file', self.cwd), capture=True, shell=True)
        preproc_env_params.assert_called_with('/tmp/y.spec.rpkg')
        tmpfile_mock.write.assert_called_with('content2')
        self.assertEqual(tmpfile.call_count, 4)
        self.assertEqual(unlink.call_count, 4)

        run.reset_mock()
        unlink.reset_mock()
        tmpfile.reset_mock()

        with open(spec_path, 'w') as spec_file:
            spec_file.write('content3')

        rpm_package = RpmPackage(self.cwd, preprocess_spec=False)
        result = rpm_package.final_spec_text()
        self.assertEqual(result, 'content3')
        run.assert_not_called()
        self.assertEqual(tmpfile.call_count, 0)
        self.assertEqual(unlink.call_count, 0)

        run.reset_mock()

        rpm_package = RpmPackage(self.cwd, 'content4', preprocess_spec=False)
        result = rpm_package.final_spec_text()
        self.assertEqual(result, 'content4')
        run.assert_not_called()

    @patch('rpkglib.RpmPackage.outdir', '/outdir')
    def test_final_spec_path(self):
        rpm_package = RpmPackage(self.cwd)

        with patch('rpkglib.RpmPackage.spec_path', '/tmp/foo.spec'):
            result = rpm_package.final_spec_path()
            self.assertEqual(result, '/outdir/foo.spec')

        with patch('rpkglib.RpmPackage.spec_path', '/tmp/bar.spec.rpkg'):
            result = rpm_package.final_spec_path()
            self.assertEqual(result, '/outdir/bar.spec')

    @patch('rpkglib.rpm_package.log')
    @patch('rpkglib.RpmPackage.final_spec_path')
    @patch('rpkglib.RpmPackage.final_spec_text')
    @patch('rpkglib.RpmPackage.setup_source_symlinks')
    def test_final_spec(self, setup_source_symlinks, final_spec_text,
                        final_spec_path, log):
        final_spec_text.return_value = 'content'
        final_spec_path.return_value = os.path.join(self.cwd, 'out.spec')
        rpm_package = RpmPackage(self.cwd)
        rpm_package.final_spec()

        with open(os.path.join(self.cwd, 'out.spec'), 'r') as f:
            self.assertEqual(f.read(), 'content\n')
        setup_source_symlinks.assert_called()
        final_spec_text.assert_called()

        setup_source_symlinks.reset_mock()
        final_spec_text.reset_mock()

        final_spec_text.return_value = 'content2'
        rpm_package = RpmPackage(self.cwd, produce_sources=False)
        rpm_package.final_spec()

        with open(os.path.join(self.cwd, 'out.spec'), 'r') as f:
            self.assertEqual(f.read(), 'content2\n')
        setup_source_symlinks.assert_not_called()
        final_spec_text.assert_called()

    @patch('rpkglib.RpmPackage.final_spec_text')
    def test_final_tmp_spec(self, final_spec_text):
        tmpdir = mkdtemp()
        final_spec_text.return_value = 'content'

        rpm_package = RpmPackage(self.cwd, base_output_path=tmpdir)
        tmp_spec_path = rpm_package.final_tmp_spec()
        tmp_spec_basename = os.path.basename(tmp_spec_path)

        with open(os.path.join(tmpdir, tmp_spec_basename), 'r') as f:
            self.assertEqual(f.read(), 'content')

        shutil.rmtree(tmpdir)

    def test_setup_source_symlinks(self):
        open('source0.tar.gz', 'w').close()
        open('patch0.txt', 'w').close()
        outdir = mkdtemp()

        rpm_package = RpmPackage(self.cwd, outdir=outdir)

        spec_path = self.dump_spec(SPEC_TEMPLATE, source0='source0.tar.gz')
        rpm_package.setup_source_symlinks(spec_path)
        result = os.listdir(outdir)[0]
        self.assertEqual('source0.tar.gz', result)
        self.assertTrue(os.path.islink(os.path.join(outdir, result)))

        spec_path = self.dump_spec(SPEC_WITH_PATCH_TEMPLATE,
                                   source0='source0.tar.gz',
                                   patch0='patch0.txt')
        rpm_package.setup_source_symlinks(spec_path)
        patch_result, source_result = sorted(os.listdir(outdir))
        self.assertEqual('source0.tar.gz', source_result)
        self.assertEqual('patch0.txt', patch_result)
        self.assertTrue(os.path.islink(os.path.join(outdir, source_result)))
        self.assertTrue(os.path.islink(os.path.join(outdir, patch_result)))

        spec_path = self.dump_spec(INVALID_SPEC_TEMPLATE)
        with pytest.raises(RpkgError):
            rpm_package.setup_source_symlinks(spec_path)

        shutil.rmtree(outdir)

    @patch('rpkglib.RpmPackage.final_tmp_spec')
    def test_clog(self, final_tmp_spec):
        rpm_package = RpmPackage(self.cwd, outdir='/tmp/outdir')

        spec_params = {
            'template': SPEC_TEMPLATE_WITH_CLOG,
            'name': 'testpkg',
            'version': 1,
            'release': 1,
            'epoch': None,
        }

        spec_path = self.dump_spec(**spec_params)
        final_tmp_spec.return_value = spec_path
        result = rpm_package.clog()
        self.assertEqual(result, 'hello world\nhow are you\nfine')

        spec_path = self.dump_spec(**spec_params)
        final_tmp_spec.return_value = spec_path
        result = rpm_package.clog(True)
        self.assertEqual(result, '- hello world\n- how are you\n  fine')

    @patch('rpkglib.RpmPackage.rpmbuild_opts')
    @patch('rpkglib.RpmPackage.final_spec')
    @patch('rpkglib.rpm_package.run')
    def test_compile(self, run, final_spec, rpmbuild_opts):
        rpm_package = RpmPackage(self.cwd)
        final_spec.return_value = '/tmp/x.spec'
        rpmbuild_opts.return_value = ['--define', '_topdir /tmp/']

        rpm_package.compile()
        run.assert_called_with(['rpmbuild', '--define', '_topdir /tmp/', '-bc',
                               '/tmp/x.spec'], capture=False)

        rpm_package.compile(target='fedora-30-x86_64', short=True, nocheck=True,
                            bcond_with=['cheese'], bcond_without=['anchovies'])
        run.assert_called_with(['rpmbuild', '--define', '_topdir /tmp/',
                                '--target', 'fedora-30-x86_64',
                                '--short-circuit', '--nocheck',
                                '--with', 'cheese', '--without', 'anchovies',
                                '-bc', '/tmp/x.spec'], capture=False)

    @patch('rpkglib.RpmPackage.rpmbuild_opts')
    @patch('rpkglib.RpmPackage.final_spec')
    @patch('rpkglib.rpm_package.run')
    def test_install(self, run, final_spec, rpmbuild_opts):
        rpm_package = RpmPackage(self.cwd)
        final_spec.return_value = '/tmp/x.spec'
        rpmbuild_opts.return_value = ['--define', '_topdir /tmp/']

        rpm_package.install()
        run.assert_called_with(['rpmbuild', '--define', '_topdir /tmp/', '-bi',
                                '/tmp/x.spec'], capture=False)

        rpm_package.install(target='fedora-30-x86_64', short=True, nocheck=True,
                            bcond_with=['cheese'], bcond_without=['anchovies'])
        run.assert_called_with(['rpmbuild', '--define', '_topdir /tmp/',
                                '--target', 'fedora-30-x86_64',
                                '--short-circuit', '--nocheck',
                                '--with', 'cheese', '--without', 'anchovies',
                                '-bi', '/tmp/x.spec'], capture=False)

    @patch('rpkglib.rpm_package.log')
    @patch('rpkglib.RpmPackage.rpmbuild_opts')
    @patch('rpkglib.RpmPackage.final_spec')
    @patch('rpkglib.rpm_package.run')
    def test_lint(self, run, final_spec, rpmbuild_opts, log):
        rpm_package = RpmPackage(self.cwd, outdir='/tmp/outdir/')
        final_spec.return_value = '/tmp/x.spec'
        rpmbuild_opts.return_value = ['--define', '_topdir /tmp/']

        rpm_package.lint()
        run.assert_called_with(['rpmlint', '--', '/tmp/outdir/'], capture=False)

        rpm_package = RpmPackage(self.cwd, outdir='/tmp/outdir/',
                                 rpmlint_config='rpmlint.conf')
        rpm_package.lint()
        run.assert_called_with(['rpmlint', '-f', 'rpmlint.conf',
                                '--', '/tmp/outdir/'], capture=False)

        rpm_package.lint(rpmlint_config='rpmlint2.conf')
        run.assert_called_with(['rpmlint', '-f', 'rpmlint2.conf',
                                '--', '/tmp/outdir/'], capture=False)

    @patch('rpkglib.RpmPackage.rpmbuild_opts')
    @patch('rpkglib.RpmPackage.final_spec')
    @patch('rpkglib.rpm_package.run')
    def test_local(self, run, final_spec, rpmbuild_opts):
        rpm_package = RpmPackage(self.cwd)
        final_spec.return_value = '/tmp/x.spec'
        rpmbuild_opts.return_value = ['--define', '_topdir /tmp/']

        rpm_package.local()
        run.assert_called_with(['rpmbuild', '--define', '_topdir /tmp/', '-ba',
                                '/tmp/x.spec'], capture=False)

        rpm_package.local(target='fedora-30-x86_64', nocheck=True,
                          bcond_with=['cheese'], bcond_without=['anchovies'])
        run.assert_called_with(['rpmbuild', '--define', '_topdir /tmp/',
                                '--target', 'fedora-30-x86_64',
                                '--nocheck', '--with', 'cheese',
                                '--without', 'anchovies',
                                '-ba', '/tmp/x.spec'], capture=False)

    @patch('rpkglib.RpmPackage.rpmbuild_opts')
    @patch('rpkglib.RpmPackage.final_spec')
    @patch('rpkglib.rpm_package.run')
    def test_prep(self, run, final_spec, rpmbuild_opts):
        rpm_package = RpmPackage(self.cwd)
        final_spec.return_value = '/tmp/x.spec'
        rpmbuild_opts.return_value = ['--define', '_topdir /tmp/']

        rpm_package.prep()
        run.assert_called_with(['rpmbuild', '--define', '_topdir /tmp/', '--nodeps',
                                '-bp', '/tmp/x.spec'], capture=False)

        rpm_package.prep(target='fedora-30-x86_64',
                         bcond_with=['cheese'], bcond_without=['anchovies'])
        run.assert_called_with(['rpmbuild', '--define', '_topdir /tmp/',
                                '--target', 'fedora-30-x86_64', '--with', 'cheese',
                                '--without', 'anchovies', '--nodeps',
                                '-bp', '/tmp/x.spec'], capture=False)

    @patch('rpkglib.RpmPackage.rpmbuild_opts')
    @patch('rpkglib.RpmPackage.final_spec')
    @patch('rpkglib.rpm_package.run')
    def test_srpm(self, run, final_spec, rpmbuild_opts):
        rpm_package = RpmPackage(self.cwd)
        final_spec.return_value = '/tmp/x.spec'
        rpmbuild_opts.return_value = ['--define', '_topdir /tmp/']

        rpm_package.srpm()
        run.assert_called_with(['rpmbuild', '--define',
                                '_topdir /tmp/', '--nodeps',
                                '-bs', '/tmp/x.spec'], capture=False)

        rpm_package.srpm(target='fedora-30-x86_64',
                         bcond_with=['cheese'], bcond_without=['anchovies'])
        run.assert_called_with(['rpmbuild', '--define', '_topdir /tmp/',
                                '--target', 'fedora-30-x86_64',
                                '--with', 'cheese',
                                '--without', 'anchovies', '--nodeps',
                                '-bs', '/tmp/x.spec'], capture=False)

    @patch('rpkglib.rpm_package.log')
    @patch('rpkglib.RpmPackage.rpmbuild_opts')
    @patch('rpkglib.RpmPackage.final_spec')
    @patch('rpkglib.rpm_package.run')
    def test_verify_files(self, run, final_spec, rpmbuild_opts, log):
        outdir = mkdtemp()

        rpm_package = RpmPackage(self.cwd, outdir=outdir)
        final_spec.return_value = '/tmp/x.spec'
        rpmbuild_opts.return_value = ['--define', '_topdir /tmp/']

        with pytest.raises(RpkgError):
            rpm_package.verify_files()

        open('{0}/x.spec'.format(outdir), 'w').close()
        rpm_package.verify_files()
        run.assert_called_with(['rpmbuild', '--define', '_topdir /tmp/', '-bl',
                                outdir+'/x.spec'], capture=False)

        rpm_package = RpmPackage(self.cwd, verbosity=utils.QUIET,
                                 outdir=outdir)
        rpm_package.verify_files()
        run.assert_called_with(['rpmbuild', '--define', '_topdir /tmp/',
                                '--quiet', '-bl', outdir+'/x.spec'], capture=False)
        shutil.rmtree(outdir)

    @patch('rpkglib.RpmPackage.final_tmp_spec')
    def test_unused_patches(self, final_tmp_spec):
        rpm_package = RpmPackage(self.cwd)

        spec_params = {
            'template': SPEC_TEMPLATE_WITH_PATCH,
            'name': 'testpkg',
            'version': 1,
            'release': 1,
            'epoch': None,
        }

        spec_path = self.dump_spec(**spec_params)
        final_tmp_spec.return_value = spec_path
        result = rpm_package.unused_patches()
        self.assertEqual(result, [])

        open('foo.patch', 'w').close()
        spec_path = self.dump_spec(**spec_params)
        final_tmp_spec.return_value = spec_path
        result = rpm_package.unused_patches()
        self.assertEqual(result, ['foo.patch'])
        os.unlink('foo.patch')

        open('hello.patch', 'w').close()
        spec_path = self.dump_spec(**spec_params)
        final_tmp_spec.return_value = spec_path
        result = rpm_package.unused_patches()
        self.assertEqual(result, [])
        os.unlink('hello.patch')
